@@ -40,6 +40,14 @@ typedef struct _GstVisualClass GstVisualClass;
 GST_DEBUG_CATEGORY_STATIC (libvisual_debug);
 #define GST_CAT_DEFAULT (libvisual_debug)
 
+/* amounf of samples before we can feed libvisual */
+#define VISUAL_SAMPLES  512
+
+#define DEFAULT_WIDTH   320
+#define DEFAULT_HEIGHT  240
+#define DEFAULT_FPS_N   25
+#define DEFAULT_FPS_D   1
+
 struct _GstVisual
 {
   GstElement element;
@@ -47,7 +55,6 @@ struct _GstVisual
   /* pads */
   GstPad *sinkpad;
   GstPad *srcpad;
-  GstClockTime next_ts;
   GstSegment segment;
 
   /* libvisual stuff */
@@ -129,6 +136,8 @@ static GstFlowReturn gst_visual_chain (GstPad * pad, GstBuffer * buffer);
 static gboolean gst_visual_sink_event (GstPad * pad, GstEvent * event);
 static gboolean gst_visual_src_event (GstPad * pad, GstEvent * event);
 
+static gboolean gst_visual_src_query (GstPad * pad, GstQuery * query);
+
 static gboolean gst_visual_sink_setcaps (GstPad * pad, GstCaps * caps);
 static gboolean gst_visual_src_setcaps (GstPad * pad, GstCaps * caps);
 static GstCaps *gst_visual_getcaps (GstPad * pad);
@@ -181,14 +190,7 @@ gst_visual_class_init (gpointer g_class, gpointer class_data)
   if (class_data == NULL) {
     parent_class = g_type_class_peek_parent (g_class);
   } else {
-    GstElementDetails details = {
-      NULL,
-      "Visualization",
-      klass->plugin->info->about,
-      "Benjamin Otte <otte@gnome.org>"
-    };
-
-    details.longname = g_strdup_printf ("libvisual %s plugin v.%s",
+    char *longname = g_strdup_printf ("libvisual %s plugin v.%s",
         klass->plugin->info->name, klass->plugin->info->version);
 
     /* FIXME: improve to only register what plugin supports? */
@@ -196,8 +198,11 @@ gst_visual_class_init (gpointer g_class, gpointer class_data)
         gst_static_pad_template_get (&src_template));
     gst_element_class_add_pad_template (element,
         gst_static_pad_template_get (&sink_template));
-    gst_element_class_set_details (element, &details);
-    g_free (details.longname);
+    gst_element_class_set_details_simple (element,
+        longname, "Visualization",
+        klass->plugin->info->about, "Benjamin Otte <otte@gnome.org>");
+
+    g_free (longname);
   }
 
   object->dispose = gst_visual_dispose;
@@ -217,6 +222,7 @@ gst_visual_init (GstVisual * visual)
   gst_pad_set_setcaps_function (visual->srcpad, gst_visual_src_setcaps);
   gst_pad_set_getcaps_function (visual->srcpad, gst_visual_getcaps);
   gst_pad_set_event_function (visual->srcpad, gst_visual_src_event);
+  gst_pad_set_query_function (visual->srcpad, gst_visual_src_query);
   gst_element_add_pad (GST_ELEMENT (visual), visual->srcpad);
 
   visual->adapter = gst_adapter_new ();
@@ -256,7 +262,6 @@ gst_visual_dispose (GObject * object)
 static void
 gst_visual_reset (GstVisual * visual)
 {
-  visual->next_ts = -1;
   gst_adapter_clear (visual->adapter);
   gst_segment_init (&visual->segment, GST_FORMAT_UNDEFINED);
 
@@ -385,7 +390,7 @@ gst_visual_sink_setcaps (GstPad * pad, GstCaps * caps)
 static gboolean
 gst_vis_src_negotiate (GstVisual * visual)
 {
-  GstCaps *othercaps, *target, *intersect;
+  GstCaps *othercaps, *target;
   GstStructure *structure;
   GstCaps *caps;
 
@@ -394,15 +399,14 @@ gst_vis_src_negotiate (GstVisual * visual)
   /* see what the peer can do */
   othercaps = gst_pad_peer_get_caps (visual->srcpad);
   if (othercaps) {
-    intersect = gst_caps_intersect (othercaps, caps);
+    target = gst_caps_intersect (othercaps, caps);
     gst_caps_unref (othercaps);
     gst_caps_unref (caps);
 
-    if (gst_caps_is_empty (intersect))
+    if (gst_caps_is_empty (target))
       goto no_format;
 
-    target = gst_caps_copy_nth (intersect, 0);
-    gst_caps_unref (intersect);
+    gst_caps_truncate (target);
   } else {
     /* need a copy, we'll be modifying it when fixating */
     target = gst_caps_copy (caps);
@@ -411,11 +415,12 @@ gst_vis_src_negotiate (GstVisual * visual)
 
   /* fixate in case something is not fixed. This does nothing if the value is
    * already fixed. For video we always try to fixate to something like
-   * 320x240x30 by convention. */
+   * 320x240x25 by convention. */
   structure = gst_caps_get_structure (target, 0);
-  gst_structure_fixate_field_nearest_int (structure, "width", 320);
-  gst_structure_fixate_field_nearest_int (structure, "height", 240);
-  gst_structure_fixate_field_nearest_fraction (structure, "framerate", 30, 1);
+  gst_structure_fixate_field_nearest_int (structure, "width", DEFAULT_WIDTH);
+  gst_structure_fixate_field_nearest_int (structure, "height", DEFAULT_HEIGHT);
+  gst_structure_fixate_field_nearest_fraction (structure, "framerate",
+      DEFAULT_FPS_N, DEFAULT_FPS_D);
 
   gst_pad_set_caps (visual->srcpad, target);
   gst_caps_unref (target);
@@ -427,7 +432,7 @@ no_format:
   {
     GST_ELEMENT_ERROR (visual, STREAM, FORMAT, (NULL),
         ("could not negotiate output format"));
-    gst_caps_unref (intersect);
+    gst_caps_unref (target);
     return FALSE;
   }
 }
@@ -520,6 +525,63 @@ gst_visual_src_event (GstPad * pad, GstEvent * event)
   return res;
 }
 
+static gboolean
+gst_visual_src_query (GstPad * pad, GstQuery * query)
+{
+  gboolean res;
+  GstVisual *visual;
+
+  visual = GST_VISUAL (gst_pad_get_parent (pad));
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_LATENCY:
+    {
+      /* We need to send the query upstream and add the returned latency to our
+       * own */
+      GstClockTime min_latency, max_latency;
+      gboolean us_live;
+      GstClockTime our_latency;
+      guint max_samples;
+
+      if ((res = gst_pad_peer_query (visual->sinkpad, query))) {
+        gst_query_parse_latency (query, &us_live, &min_latency, &max_latency);
+
+        GST_DEBUG_OBJECT (visual, "Peer latency: min %"
+            GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (min_latency), GST_TIME_ARGS (max_latency));
+
+        /* the max samples we must buffer buffer */
+        max_samples = MAX (VISUAL_SAMPLES, visual->spf);
+        our_latency =
+            gst_util_uint64_scale_int (max_samples, GST_SECOND, visual->rate);
+
+        GST_DEBUG_OBJECT (visual, "Our latency: %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (our_latency));
+
+        /* we add some latency but only if we need to buffer more than what
+         * upstream gives us */
+        min_latency += our_latency;
+        if (max_latency != -1)
+          max_latency += our_latency;
+
+        GST_DEBUG_OBJECT (visual, "Calculated total latency : min %"
+            GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (min_latency), GST_TIME_ARGS (max_latency));
+
+        gst_query_set_latency (query, TRUE, min_latency, max_latency);
+      }
+      break;
+    }
+    default:
+      res = gst_pad_peer_query (visual->sinkpad, query);
+      break;
+  }
+
+  gst_object_unref (visual);
+
+  return res;
+}
+
 /* allocate and output buffer, if no format was negotiated, this
  * function will negotiate one. After calling this function, a
  * reverse negotiation could have happened. */
@@ -576,13 +638,8 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
   /* resync on DISCONT */
   if (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)) {
     gst_adapter_clear (visual->adapter);
-    visual->next_ts = -1;
   }
 
-  /* Match timestamps from the incoming audio */
-  if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE)
-    visual->next_ts = GST_BUFFER_TIMESTAMP (buffer);
-
   GST_DEBUG_OBJECT (visual,
       "Input buffer has %d samples, time=%" G_GUINT64_FORMAT,
       GST_BUFFER_SIZE (buffer) / visual->bps, GST_BUFFER_TIMESTAMP (buffer));
@@ -592,26 +649,36 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
   while (TRUE) {
     gboolean need_skip;
     const guint16 *data;
+    guint64 dist, timestamp;
 
     GST_DEBUG_OBJECT (visual, "processing buffer");
 
     avail = gst_adapter_available (visual->adapter);
     GST_DEBUG_OBJECT (visual, "avail now %u", avail);
 
-    /* we need at least 512 samples */
-    if (avail < 512 * visual->bps)
+    /* we need at least VISUAL_SAMPLES samples */
+    if (avail < VISUAL_SAMPLES * visual->bps)
       break;
 
     /* we need at least enough samples to make one frame */
     if (avail < visual->spf * visual->bps)
       break;
 
-    if (visual->next_ts != -1) {
+    /* get timestamp of the current adapter byte */
+    timestamp = gst_adapter_prev_timestamp (visual->adapter, &dist);
+    if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
+      /* convert bytes to time */
+      dist /= visual->bps;
+      timestamp += gst_util_uint64_scale_int (dist, GST_SECOND, visual->rate);
+    }
+
+    if (timestamp != -1) {
       gint64 qostime;
 
       /* QoS is done on running time */
       qostime = gst_segment_to_running_time (&visual->segment, GST_FORMAT_TIME,
-          visual->next_ts);
+          timestamp);
+      qostime += visual->duration;
 
       GST_OBJECT_LOCK (visual);
       /* check for QoS, don't compute buffers that are known to be late */
@@ -627,26 +694,27 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
       }
     }
 
-    /* Read 512 samples per channel */
+    /* Read VISUAL_SAMPLES samples per channel */
     data =
-        (const guint16 *) gst_adapter_peek (visual->adapter, 512 * visual->bps);
+        (const guint16 *) gst_adapter_peek (visual->adapter,
+        VISUAL_SAMPLES * visual->bps);
 
 #if defined(VISUAL_API_VERSION) && VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
     {
       VisBuffer *lbuf, *rbuf;
-      guint16 ldata[512], rdata[512];
+      guint16 ldata[VISUAL_SAMPLES], rdata[VISUAL_SAMPLES];
       VisAudioSampleRateType rate;
 
       lbuf = visual_buffer_new_with_buffer (ldata, sizeof (ldata), NULL);
       rbuf = visual_buffer_new_with_buffer (rdata, sizeof (rdata), NULL);
 
       if (visual->channels == 2) {
-        for (i = 0; i < 512; i++) {
+        for (i = 0; i < VISUAL_SAMPLES; i++) {
           ldata[i] = *data++;
           rdata[i] = *data++;
         }
       } else {
-        for (i = 0; i < 512; i++) {
+        for (i = 0; i < VISUAL_SAMPLES; i++) {
           ldata[i] = *data;
           rdata[i] = *data++;
         }
@@ -685,10 +753,11 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
 
       visual_audio_samplepool_input_channel (visual->audio->samplepool,
           lbuf,
-          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_CHANNEL_LEFT);
-      visual_audio_samplepool_input_channel (visual->audio->samplepool,
-          rbuf,
-          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16, VISUAL_AUDIO_CHANNEL_RIGHT);
+          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16,
+          (char *) VISUAL_AUDIO_CHANNEL_LEFT);
+      visual_audio_samplepool_input_channel (visual->audio->samplepool, rbuf,
+          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16,
+          (char *) VISUAL_AUDIO_CHANNEL_RIGHT);
 
       visual_object_unref (VISUAL_OBJECT (lbuf));
       visual_object_unref (VISUAL_OBJECT (rbuf));
@@ -696,12 +765,12 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     }
 #else
     if (visual->channels == 2) {
-      for (i = 0; i < 512; i++) {
+      for (i = 0; i < VISUAL_SAMPLES; i++) {
         visual->audio->plugpcm[0][i] = *data++;
         visual->audio->plugpcm[1][i] = *data++;
       }
     } else {
-      for (i = 0; i < 512; i++) {
+      for (i = 0; i < VISUAL_SAMPLES; i++) {
         visual->audio->plugpcm[0][i] = *data;
         visual->audio->plugpcm[1][i] = *data++;
       }
@@ -722,17 +791,13 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     visual_video_set_buffer (visual->video, NULL);
     GST_DEBUG_OBJECT (visual, "rendered one frame");
 
-    GST_BUFFER_TIMESTAMP (outbuf) = visual->next_ts;
+    GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
     GST_BUFFER_DURATION (outbuf) = visual->duration;
 
     ret = gst_pad_push (visual->srcpad, outbuf);
     outbuf = NULL;
 
   skip:
-    /* interpollate next timestamp */
-    if (visual->next_ts != -1)
-      visual->next_ts += visual->duration;
-
     GST_DEBUG_OBJECT (visual, "finished frame, flushing %u samples from input",
         visual->spf);
 
