@@ -72,6 +72,8 @@ static gboolean vorbis_dec_sink_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn vorbis_dec_chain (GstPad * pad, GstBuffer * buffer);
 static GstFlowReturn vorbis_dec_chain_forward (GstVorbisDec * vd,
     gboolean discont, GstBuffer * buffer);
+static GstFlowReturn vorbis_dec_chain_reverse (GstVorbisDec * vd,
+    gboolean discont, GstBuffer * buf);
 static GstStateChangeReturn vorbis_dec_change_state (GstElement * element,
     GstStateChange transition);
 
@@ -164,7 +166,10 @@ vorbis_dec_finalize (GObject * object)
    */
   GstVorbisDec *vd = GST_VORBIS_DEC (object);
 
+#ifndef USE_TREMOLO
   vorbis_block_clear (&vd->vb);
+#endif
+
   vorbis_dsp_clear (&vd->vd);
   vorbis_comment_clear (&vd->vc);
   vorbis_info_clear (&vd->vi);
@@ -292,6 +297,8 @@ vorbis_dec_src_query (GstPad * pad, GstQuery * query)
   gboolean res = FALSE;
 
   dec = GST_VORBIS_DEC (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (dec == NULL))
+    return FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:
@@ -406,6 +413,10 @@ vorbis_dec_src_event (GstPad * pad, GstEvent * event)
   GstVorbisDec *dec;
 
   dec = GST_VORBIS_DEC (gst_pad_get_parent (pad));
+  if (G_UNLIKELY (dec == NULL)) {
+    gst_event_unref (event);
+    return FALSE;
+  }
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
@@ -467,6 +478,8 @@ vorbis_dec_sink_event (GstPad * pad, GstEvent * event)
   GST_LOG_OBJECT (dec, "handling event");
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
+      if (dec->segment.rate < 0.0)
+        vorbis_dec_chain_reverse (dec, TRUE, NULL);
       ret = gst_pad_push_event (dec->srcpad, event);
       break;
     case GST_EVENT_FLUSH_START:
@@ -588,11 +601,18 @@ vorbis_handle_identification_packet (GstVorbisDec * vd)
       s = gst_caps_get_structure (caps, 0);
       /* template ensures 16 or 32 */
       gst_structure_get_int (s, "width", &width);
+
+      GST_INFO_OBJECT (vd, "using %s with %d channels and %d bit audio depth",
+          gst_structure_get_name (s), vd->vi.channels, width);
     }
     gst_caps_unref (caps);
   }
   vd->width = width >> 3;
 
+  /* select a copy_samples function, this way we can have specialized versions
+   * for mono/stereo and avoid the depth switch in tremor case */
+  vd->copy_samples = get_copy_sample_func (vd->vi.channels, vd->width);
+
   caps = gst_caps_copy (gst_pad_get_pad_template_caps (vd->srcpad));
   gst_caps_set_simple (caps, "rate", G_TYPE_INT, vd->vi.rate,
       "channels", G_TYPE_INT, vd->vi.channels,
@@ -695,11 +715,16 @@ vorbis_handle_type_packet (GstVorbisDec * vd)
 
   g_assert (vd->initialized == FALSE);
 
+#ifdef USE_TREMOLO
+  if (G_UNLIKELY ((res = vorbis_dsp_init (&vd->vd, &vd->vi))))
+    goto synthesis_init_error;
+#else
   if (G_UNLIKELY ((res = vorbis_synthesis_init (&vd->vd, &vd->vi))))
     goto synthesis_init_error;
 
   if (G_UNLIKELY ((res = vorbis_block_init (&vd->vd, &vd->vb))))
     goto block_init_error;
+#endif
 
   vd->initialized = TRUE;
 
@@ -743,7 +768,11 @@ vorbis_handle_header_packet (GstVorbisDec * vd, ogg_packet * packet)
   /* Packetno = 0 if the first byte is exactly 0x01 */
   packet->b_o_s = ((gst_ogg_packet_data (packet))[0] == 0x1) ? 1 : 0;
 
+#ifdef USE_TREMOLO
+  if ((ret = vorbis_dsp_headerin (&vd->vi, &vd->vc, packet)))
+#else
   if ((ret = vorbis_synthesis_headerin (&vd->vi, &vd->vc, packet)))
+#endif
     goto header_read_error;
 
   switch ((gst_ogg_packet_data (packet))[0]) {
@@ -816,7 +845,7 @@ vorbis_do_timestamps (GstVorbisDec * vd, GstBuffer * buf, gboolean reverse,
     GstClockTime timestamp, GstClockTime duration)
 {
   /* interpolate reverse */
-  if (vd->last_timestamp != -1 && reverse)
+  if (vd->last_timestamp != -1 && duration != -1 && reverse)
     vd->last_timestamp -= duration;
 
   /* take buffer timestamp, use interpolated timestamp otherwise */
@@ -826,20 +855,31 @@ vorbis_do_timestamps (GstVorbisDec * vd, GstBuffer * buf, gboolean reverse,
     timestamp = vd->last_timestamp;
 
   /* interpolate forwards */
-  if (vd->last_timestamp != -1 && !reverse)
+  if (vd->last_timestamp != -1 && duration != -1 && !reverse)
     vd->last_timestamp += duration;
 
-  GST_BUFFER_TIMESTAMP (buf) = timestamp;
-  GST_BUFFER_DURATION (buf) = duration;
+  GST_LOG_OBJECT (vd,
+      "keeping timestamp %" GST_TIME_FORMAT " ts %" GST_TIME_FORMAT " dur %"
+      GST_TIME_FORMAT, GST_TIME_ARGS (vd->last_timestamp),
+      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (duration));
+
+  if (buf) {
+    GST_BUFFER_TIMESTAMP (buf) = timestamp;
+    GST_BUFFER_DURATION (buf) = duration;
+  }
 }
 
 static GstFlowReturn
 vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet,
     GstClockTime timestamp, GstClockTime duration)
 {
+#ifdef USE_TREMOLO
+  vorbis_sample_t *pcm;
+#else
   vorbis_sample_t **pcm;
+#endif
   guint sample_count;
-  GstBuffer *out;
+  GstBuffer *out = NULL;
   GstFlowReturn result;
   gint size;
 
@@ -852,17 +892,27 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet,
    * packet to decode the current one so we must be carefull not to
    * throw away too much. For now we decode everything and clip right
    * before pushing data. */
+
+#ifdef USE_TREMOLO
+  if (G_UNLIKELY (vorbis_dsp_synthesis (&vd->vd, packet, 1)))
+    goto could_not_read;
+#else
   if (G_UNLIKELY (vorbis_synthesis (&vd->vb, packet)))
     goto could_not_read;
 
   if (G_UNLIKELY (vorbis_synthesis_blockin (&vd->vd, &vd->vb) < 0))
     goto not_accepted;
+#endif
 
   /* assume all goes well here */
   result = GST_FLOW_OK;
 
   /* count samples ready for reading */
+#ifdef USE_TREMOLO
+  if ((sample_count = vorbis_dsp_pcmout (&vd->vd, NULL, 0)) == 0)
+#else
   if ((sample_count = vorbis_synthesis_pcmout (&vd->vd, NULL)) == 0)
+#endif
     goto done;
 
   size = sample_count * vd->vi.channels * vd->width;
@@ -877,12 +927,19 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet,
     goto done;
 
   /* get samples ready for reading now, should be sample_count */
+#ifdef USE_TREMOLO
+  pcm = GST_BUFFER_DATA (out);
+  if (G_UNLIKELY ((vorbis_dsp_pcmout (&vd->vd, pcm, sample_count)) != sample_count))
+#else
   if (G_UNLIKELY ((vorbis_synthesis_pcmout (&vd->vd, &pcm)) != sample_count))
+#endif
     goto wrong_samples;
 
+#ifndef USE_TREMOLO
   /* copy samples in buffer */
-  copy_samples ((vorbis_sample_t *) GST_BUFFER_DATA (out), pcm, sample_count,
-      vd->vi.channels, vd->width);
+  vd->copy_samples ((vorbis_sample_t *) GST_BUFFER_DATA (out), pcm,
+      sample_count, vd->vi.channels, vd->width);
+#endif
 
   GST_LOG_OBJECT (vd, "setting output size to %d", size);
   GST_BUFFER_SIZE (out) = size;
@@ -899,7 +956,16 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet,
     result = vorbis_dec_push_reverse (vd, out);
 
 done:
+  if (out == NULL) {
+    /* no output, still keep track of timestamps */
+    vorbis_do_timestamps (vd, NULL, FALSE, timestamp, duration);
+  }
+
+#ifdef USE_TREMOLO
+  vorbis_dsp_read (&vd->vd, sample_count);
+#else
   vorbis_synthesis_read (&vd->vd, sample_count);
+#endif
 
   return result;
 
@@ -1128,13 +1194,16 @@ vorbis_dec_chain_reverse (GstVorbisDec * vd, gboolean discont, GstBuffer * buf)
     result = vorbis_dec_flush_decode (vd);
   }
 
-  GST_DEBUG_OBJECT (vd, "gathering buffer %p of size %u, time %" GST_TIME_FORMAT
-      ", dur %" GST_TIME_FORMAT, buf, GST_BUFFER_SIZE (buf),
-      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-      GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
+  if (G_LIKELY (buf)) {
+    GST_DEBUG_OBJECT (vd,
+        "gathering buffer %p of size %u, time %" GST_TIME_FORMAT
+        ", dur %" GST_TIME_FORMAT, buf, GST_BUFFER_SIZE (buf),
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+        GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
 
-  /* add buffer to gather queue */
-  vd->gather = g_list_prepend (vd->gather, buf);
+    /* add buffer to gather queue */
+    vd->gather = g_list_prepend (vd->gather, buf);
+  }
 
   return result;
 }
@@ -1212,7 +1281,11 @@ vorbis_dec_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       GST_DEBUG_OBJECT (vd, "PAUSED -> READY, clearing vorbis structures");
       vd->initialized = FALSE;
+
+#ifndef USE_TREMOLO
       vorbis_block_clear (&vd->vb);
+#endif
+
       vorbis_dsp_clear (&vd->vd);
       vorbis_comment_clear (&vd->vc);
       vorbis_info_clear (&vd->vi);
