@@ -55,6 +55,7 @@ typedef struct _GstXOverlayClass GstXOverlayClass;
  * @set_xwindow_id: virtual method to configure the XWindow id
  * @expose: virtual method to handle expose events
  * @handle_events: virtual method to handle events
+ * @set_render_rectangle: virtual method to set the render rectangle (since 0.10.29)
  *
  * #GstXOverlay interface
  */
@@ -62,30 +63,56 @@ struct _GstXOverlayClass {
   GTypeInterface klass;
 
   /* virtual functions */
-  void (* set_xwindow_id) (GstXOverlay *overlay,
-                           gulong       xwindow_id);
+#ifndef GST_DISABLE_DEPRECATED
+  void (* set_xwindow_id)      (GstXOverlay *overlay,
+                                gulong       xwindow_id);
+#else
+  void (* set_xwindow_id_disabled) (GstXOverlay *overlay,
+                                gulong       xwindow_id);
+#endif /* not GST_DISABLE_DEPRECATED */
 
-  void (* expose)         (GstXOverlay *overlay);
+  void (* expose)              (GstXOverlay *overlay);
   
-  void (* handle_events)  (GstXOverlay *overlay,
-                           gboolean     handle_events);  
+  void (* handle_events)       (GstXOverlay *overlay,
+                                gboolean     handle_events);  
 
-  /*< private >*/
-  gpointer                 _gst_reserved[GST_PADDING - 1];
+  void (* set_render_rectangle) (GstXOverlay *overlay,
+                                 gint x, gint y,
+                                 gint width, gint height);
+
+  void (* set_window_handle)   (GstXOverlay *overlay,
+                                guintptr    handle);
+    /*< private >*/
+  gpointer                 _gst_reserved[GST_PADDING - 3];
 };
 
 GType   gst_x_overlay_get_type          (void);
 
 /* virtual class function wrappers */
-void gst_x_overlay_set_xwindow_id     (GstXOverlay *overlay, gulong xwindow_id);
+#ifndef GST_DISABLE_DEPRECATED
+void gst_x_overlay_set_xwindow_id      (GstXOverlay *overlay, 
+                                        gulong xwindow_id);
+#endif
 
-void gst_x_overlay_expose             (GstXOverlay *overlay);
+gboolean gst_x_overlay_set_render_rectangle (GstXOverlay *overlay,
+                                             gint x, gint y,
+                                             gint width, gint height);
 
-void gst_x_overlay_handle_events      (GstXOverlay *overlay,
-                                       gboolean     handle_events);
+void gst_x_overlay_expose              (GstXOverlay *overlay);
+
+void gst_x_overlay_handle_events       (GstXOverlay *overlay,
+                                        gboolean     handle_events);
+
+void gst_x_overlay_set_window_handle   (GstXOverlay *overlay, 
+                                        guintptr handle);
 
 /* public methods to dispatch bus messages */
+#ifndef GST_DISABLE_DEPRECATED
 void gst_x_overlay_got_xwindow_id     (GstXOverlay *overlay, gulong xwindow_id);
+#endif
+
+void gst_x_overlay_got_window_handle  (GstXOverlay *overlay,
+    guintptr handle);
 
 void gst_x_overlay_prepare_xwindow_id (GstXOverlay *overlay);
 
