@@ -453,33 +453,20 @@ gst_rtp_buffer_list_validate (GstBufferList * list)
     guint8 *packet_payload;
     guint payload_size;
     guint packet_size;
+    guint j, n_buffers;
 
-    /* each group should consists of 2 buffers: one containing the RTP header
-     * and the other one the payload, FIXME, relax the requirement of only one
-     * payload buffer. */
-    if (gst_buffer_list_iterator_n_buffers (it) != 2)
+    /* each group should consists of at least 1 buffer: The first buffer always
+     * contains the complete RTP header. Next buffers contain the payload */
+    n_buffers = gst_buffer_list_iterator_n_buffers (it);
+    if (n_buffers < 1)
       goto invalid_list;
 
-    /* get the RTP header */
+    /* get the RTP header (and if n_buffers == 1 also the payload) */
     rtpbuf = gst_buffer_list_iterator_next (it);
     packet_header = GST_BUFFER_DATA (rtpbuf);
     if (packet_header == NULL)
       goto invalid_list;
 
-    /* get the payload */
-    paybuf = gst_buffer_list_iterator_next (it);
-    packet_payload = GST_BUFFER_DATA (paybuf);
-    if (packet_payload == NULL) {
-      goto invalid_list;
-    }
-    payload_size = GST_BUFFER_SIZE (paybuf);
-    if (payload_size == 0) {
-      goto invalid_list;
-    }
-
-    /* the size of the RTP packet within the current group */
-    packet_size = GST_BUFFER_SIZE (rtpbuf) + payload_size;
-
     /* check the sequence number */
     if (G_UNLIKELY (i == 0)) {
       prev_seqnum = g_ntohs (GST_RTP_HEADER_SEQ (packet_header));
@@ -489,6 +476,25 @@ gst_rtp_buffer_list_validate (GstBufferList * list)
         goto invalid_list;
     }
 
+    packet_size = GST_BUFFER_SIZE (rtpbuf);
+    packet_payload = NULL;
+    payload_size = 0;
+
+    /* get the payload buffers */
+    for (j = 1; j < n_buffers; j++) {
+      /* get the payload */
+      paybuf = gst_buffer_list_iterator_next (it);
+
+      if ((packet_payload = GST_BUFFER_DATA (paybuf)) == NULL)
+        goto invalid_list;
+
+      if ((payload_size = GST_BUFFER_SIZE (paybuf)) == 0)
+        goto invalid_list;
+
+      /* the size of the RTP packet within the current group */
+      packet_size += payload_size;
+    }
+
     /* validate packet */
     if (!validate_data (packet_header, packet_size, packet_payload,
             payload_size)) {
@@ -739,7 +745,7 @@ gst_rtp_buffer_get_extension_data (GstBuffer * buffer, guint16 * bits,
  *
  * Returns: True if done.
  *
- * Since : 0.10.18
+ * Since: 0.10.18
  */
 gboolean
 gst_rtp_buffer_set_extension_data (GstBuffer * buffer, guint16 bits,
@@ -1608,7 +1614,6 @@ gst_rtp_buffer_add_extension_onebyte_header (GstBuffer * buffer, guint8 id,
   guint8 *pdata;
   guint wordlen;
   gboolean has_bit;
-  guint bytelen;
 
   g_return_val_if_fail (id > 0 && id < 15, FALSE);
   g_return_val_if_fail (size >= 1 && size <= 16, FALSE);
@@ -1617,8 +1622,6 @@ gst_rtp_buffer_add_extension_onebyte_header (GstBuffer * buffer, guint8 id,
   has_bit = gst_rtp_buffer_get_extension_data (buffer, &bits,
       (gpointer) & pdata, &wordlen);
 
-  bytelen = wordlen * 4;
-
   if (has_bit) {
     gulong offset = 0;
     guint8 *nextext;
@@ -1732,7 +1735,6 @@ gst_rtp_buffer_add_extension_twobytes_header (GstBuffer * buffer,
   guint8 *pdata;
   guint wordlen;
   gboolean has_bit;
-  guint bytelen;
 
   g_return_val_if_fail ((appbits & 0xF0) == 0, FALSE);
   g_return_val_if_fail (size < 256, FALSE);
@@ -1741,8 +1743,6 @@ gst_rtp_buffer_add_extension_twobytes_header (GstBuffer * buffer,
   has_bit = gst_rtp_buffer_get_extension_data (buffer, &bits,
       (gpointer) & pdata, &wordlen);
 
-  bytelen = wordlen * 4;
-
   if (has_bit) {
     gulong offset = 0;
     guint8 *nextext;
@@ -1776,7 +1776,7 @@ gst_rtp_buffer_add_extension_twobytes_header (GstBuffer * buffer,
     gst_rtp_buffer_set_extension_data (buffer, (0x100 << 4) | (appbits & 0x0F),
         wordlen);
   } else {
-    wordlen = (size + 1) / 4 + (((size + 1) % 4) ? 1 : 0);
+    wordlen = (size + 2) / 4 + (((size + 2) % 4) ? 1 : 0);
 
     gst_rtp_buffer_set_extension_data (buffer, (0x100 << 4) | (appbits & 0x0F),
         wordlen);
