@@ -97,10 +97,6 @@ speex_free (void *ptr)
 
 #include <math.h>
 
-#ifndef M_PI
-#define M_PI 3.14159263
-#endif
-
 #ifdef FIXED_POINT
 #define WORD2INT(x) ((x) < -32767 ? -32768 : ((x) > 32766 ? 32767 : (x)))
 #else
@@ -323,7 +319,7 @@ sinc (float cutoff, float x, int N, struct FuncDef *window_func)
   else if (fabs (x) > .5f * N)
     return 0;
   /*FIXME: Can it really be any slower than this? */
-  return WORD2INT (32768. * cutoff * sin (M_PI * xx) / (M_PI * xx) *
+  return WORD2INT (32768. * cutoff * sin (G_PI * xx) / (G_PI * xx) *
       compute_func (fabs (2. * x / N), window_func));
 }
 #else
@@ -346,7 +342,7 @@ sinc (float cutoff, float x, int N, struct FuncDef *window_func)
   else if (fabs (x) > .5 * N)
     return 0;
   /*FIXME: Can it really be any slower than this? */
-  return cutoff * sin (M_PI * xx) / (M_PI * xx) * compute_func (fabs (2. * x /
+  return cutoff * sin (G_PI * xx) / (G_PI * xx) * compute_func (fabs (2. * x /
           N), window_func);
 }
 #endif
@@ -415,20 +411,27 @@ resampler_basic_direct_single (SpeexResamplerState * st,
     const spx_word16_t *iptr = &in[last_sample];
 
 #ifndef OVERRIDE_INNER_PRODUCT_SINGLE
-    float accum[4] = { 0, 0, 0, 0 };
+    sum = 0;
+    for (j = 0; j < N; j++)
+      sum += MULT16_16 (sinc[j], iptr[j]);
 
-    for (j = 0; j < N; j += 4) {
-      accum[0] += sinc[j] * iptr[j];
-      accum[1] += sinc[j + 1] * iptr[j + 1];
-      accum[2] += sinc[j + 2] * iptr[j + 2];
-      accum[3] += sinc[j + 3] * iptr[j + 3];
-    }
-    sum = accum[0] + accum[1] + accum[2] + accum[3];
+/*    This code is slower on most DSPs which have only 2 accumulators.
+      Plus this this forces truncation to 32 bits and you lose the HW guard bits.
+      I think we can trust the compiler and let it vectorize and/or unroll itself.
+      spx_word32_t accum[4] = {0,0,0,0};
+      for(j=0;j<N;j+=4) {
+        accum[0] += MULT16_16(sinc[j], iptr[j]);
+        accum[1] += MULT16_16(sinc[j+1], iptr[j+1]);
+        accum[2] += MULT16_16(sinc[j+2], iptr[j+2]);
+        accum[3] += MULT16_16(sinc[j+3], iptr[j+3]);
+      }
+      sum = accum[0] + accum[1] + accum[2] + accum[3];
+*/
 #else
     sum = inner_product_single (sinc, iptr, N);
 #endif
 
-    out[out_stride * out_sample++] = PSHR32 (sum, 15);
+    out[out_stride * out_sample++] = SATURATE32 (PSHR32 (sum, 15), 32767);
     last_sample += int_advance;
     samp_frac_num += frac_advance;
     if (samp_frac_num >= den_rate) {
@@ -552,9 +555,10 @@ resampler_basic_interpolate_single (SpeexResamplerState * st,
 
     cubic_coef (frac, interp);
     sum =
-        MULT16_32_Q15 (interp[0], accum[0]) + MULT16_32_Q15 (interp[1],
-        accum[1]) + MULT16_32_Q15 (interp[2],
-        accum[2]) + MULT16_32_Q15 (interp[3], accum[3]);
+        MULT16_32_Q15 (interp[0], SHR32 (accum[0],
+            1)) + MULT16_32_Q15 (interp[1], SHR32 (accum[1],
+            1)) + MULT16_32_Q15 (interp[2], SHR32 (accum[2],
+            1)) + MULT16_32_Q15 (interp[3], SHR32 (accum[3], 1));
 #else
     cubic_coef (frac, interp);
     sum =
@@ -563,7 +567,7 @@ resampler_basic_interpolate_single (SpeexResamplerState * st,
         interp);
 #endif
 
-    out[out_stride * out_sample++] = PSHR32 (sum, 15);
+    out[out_stride * out_sample++] = SATURATE32 (PSHR32 (sum, 14), 32767);
     last_sample += int_advance;
     samp_frac_num += frac_advance;
     if (samp_frac_num >= den_rate) {
@@ -1302,6 +1306,12 @@ speex_resampler_get_output_latency (SpeexResamplerState * st)
       (st->num_rate >> 1)) / st->num_rate;
 }
 
+EXPORT int
+speex_resampler_get_filt_len (SpeexResamplerState * st)
+{
+  return st->filt_len;
+}
+
 EXPORT int
 speex_resampler_skip_zeros (SpeexResamplerState * st)
 {
