@@ -48,25 +48,11 @@
 #include "gstaudiotestsrc.h"
 
 
-#ifndef M_PI
-#define M_PI  3.14159265358979323846
-#endif
-
-#ifndef M_PI_2
-#define M_PI_2  1.57079632679489661923
-#endif
-
-#define M_PI_M2 ( M_PI + M_PI )
+#define M_PI_M2 ( G_PI + G_PI )
 
 GST_DEBUG_CATEGORY_STATIC (audio_test_src_debug);
 #define GST_CAT_DEFAULT audio_test_src_debug
 
-static const GstElementDetails gst_audio_test_src_details =
-GST_ELEMENT_DETAILS ("Audio test source",
-    "Source/Audio",
-    "Creates audio test signals of given frequency and volume",
-    "Stefan Kost <ensonic@users.sf.net>");
-
 #define DEFAULT_SAMPLES_PER_BUFFER   1024
 #define DEFAULT_WAVE                 GST_AUDIO_TEST_SRC_WAVE_SINE
 #define DEFAULT_FREQ                 440.0
@@ -146,6 +132,8 @@ gst_audiostestsrc_wave_get_type (void)
   return audiostestsrc_wave_type;
 }
 
+static void gst_audio_test_src_finalize (GObject * object);
+
 static void gst_audio_test_src_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
 static void gst_audio_test_src_get_property (GObject * object,
@@ -179,7 +167,10 @@ gst_audio_test_src_base_init (gpointer g_class)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_audio_test_src_src_template));
-  gst_element_class_set_details (element_class, &gst_audio_test_src_details);
+  gst_element_class_set_details_simple (element_class,
+      "Audio test source", "Source/Audio",
+      "Creates audio test signals of given frequency and volume",
+      "Stefan Kost <ensonic@users.sf.net>");
 }
 
 static void
@@ -193,6 +184,7 @@ gst_audio_test_src_class_init (GstAudioTestSrcClass * klass)
 
   gobject_class->set_property = gst_audio_test_src_set_property;
   gobject_class->get_property = gst_audio_test_src_get_property;
+  gobject_class->finalize = gst_audio_test_src_finalize;
 
   g_object_class_install_property (gobject_class, PROP_SAMPLES_PER_BUFFER,
       g_param_spec_int ("samplesperbuffer", "Samples per buffer",
@@ -266,10 +258,24 @@ gst_audio_test_src_init (GstAudioTestSrc * src, GstAudioTestSrcClass * g_class)
   src->timestamp_offset = DEFAULT_TIMESTAMP_OFFSET;
   src->can_activate_pull = DEFAULT_CAN_ACTIVATE_PULL;
 
+  src->gen = NULL;
+
   src->wave = DEFAULT_WAVE;
   gst_base_src_set_blocksize (GST_BASE_SRC (src), -1);
 }
 
+static void
+gst_audio_test_src_finalize (GObject * object)
+{
+  GstAudioTestSrc *src = GST_AUDIO_TEST_SRC (object);
+
+  if (src->gen)
+    g_rand_free (src->gen);
+  src->gen = NULL;
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 gst_audio_test_src_src_fixate (GstPad * pad, GstCaps * caps)
 {
@@ -462,7 +468,7 @@ gst_audio_test_src_create_square_##type (GstAudioTestSrc * src, g##type * sample
       src->accumulator -= M_PI_M2; \
     \
     for (c = 0; c < src->channels; ++c) { \
-      samples[i++] = (g##type) ((src->accumulator < M_PI) ? amp : -amp); \
+      samples[i++] = (g##type) ((src->accumulator < G_PI) ? amp : -amp); \
     } \
   } \
 }
@@ -487,7 +493,7 @@ gst_audio_test_src_create_saw_##type (GstAudioTestSrc * src, g##type * samples)
   gdouble step, amp; \
   \
   step = M_PI_M2 * src->freq / src->samplerate; \
-  amp = (src->volume * scale) / M_PI; \
+  amp = (src->volume * scale) / G_PI; \
   \
   i = 0; \
   while (i < (src->generate_samples_per_buffer * src->channels)) { \
@@ -495,7 +501,7 @@ gst_audio_test_src_create_saw_##type (GstAudioTestSrc * src, g##type * samples)
     if (src->accumulator >= M_PI_M2) \
       src->accumulator -= M_PI_M2; \
     \
-    if (src->accumulator < M_PI) { \
+    if (src->accumulator < G_PI) { \
       for (c = 0; c < src->channels; ++c) \
         samples[i++] = (g##type) (src->accumulator * amp); \
     } else { \
@@ -525,7 +531,7 @@ gst_audio_test_src_create_triangle_##type (GstAudioTestSrc * src, g##type * samp
   gdouble step, amp; \
   \
   step = M_PI_M2 * src->freq / src->samplerate; \
-  amp = (src->volume * scale) / M_PI_2; \
+  amp = (src->volume * scale) / G_PI_2; \
   \
   i = 0; \
   while (i < (src->generate_samples_per_buffer * src->channels)) { \
@@ -533,12 +539,12 @@ gst_audio_test_src_create_triangle_##type (GstAudioTestSrc * src, g##type * samp
     if (src->accumulator >= M_PI_M2) \
       src->accumulator -= M_PI_M2; \
     \
-    if (src->accumulator < (M_PI * 0.5)) { \
+    if (src->accumulator < (G_PI_2)) { \
       for (c = 0; c < src->channels; ++c) \
         samples[i++] = (g##type) (src->accumulator * amp); \
-    } else if (src->accumulator < (M_PI * 1.5)) { \
+    } else if (src->accumulator < (G_PI * 1.5)) { \
       for (c = 0; c < src->channels; ++c) \
-        samples[i++] = (g##type) ((src->accumulator - M_PI) * -amp); \
+        samples[i++] = (g##type) ((src->accumulator - G_PI) * -amp); \
     } else { \
       for (c = 0; c < src->channels; ++c) \
         samples[i++] = (g##type) ((M_PI_M2 - src->accumulator) * -amp); \
@@ -587,7 +593,7 @@ gst_audio_test_src_create_white_noise_##type (GstAudioTestSrc * src, g##type * s
   i = 0; \
   while (i < (src->generate_samples_per_buffer * src->channels)) { \
     for (c = 0; c < src->channels; ++c) \
-      samples[i++] = (g##type) (amp * g_random_double_range (-1.0, 1.0)); \
+      samples[i++] = (g##type) (amp * g_rand_double_range (src->gen, -1.0, 1.0)); \
   } \
 }
 
@@ -629,8 +635,9 @@ gst_audio_test_src_init_pink_noise (GstAudioTestSrc * src)
 
 /* Generate Pink noise values between -1.0 and +1.0 */
 static gdouble
-gst_audio_test_src_generate_pink_noise_value (GstPinkNoise * pink)
+gst_audio_test_src_generate_pink_noise_value (GstAudioTestSrc * src)
 {
+  GstPinkNoise *pink = &src->pink;
   glong new_random;
   glong sum;
 
@@ -654,13 +661,15 @@ gst_audio_test_src_generate_pink_noise_value (GstPinkNoise * pink)
      * values together. Only one changes each time.
      */
     pink->running_sum -= pink->rows[num_zeros];
-    new_random = 32768.0 - (65536.0 * (gulong) rand () / (RAND_MAX + 1.0));
+    new_random = 32768.0 - (65536.0 * (gulong) g_rand_int (src->gen)
+        / (G_MAXUINT32 + 1.0));
     pink->running_sum += new_random;
     pink->rows[num_zeros] = new_random;
   }
 
   /* Add extra white noise value. */
-  new_random = 32768.0 - (65536.0 * (gulong) rand () / (RAND_MAX + 1.0));
+  new_random = 32768.0 - (65536.0 * (gulong) g_rand_int (src->gen)
+      / (G_MAXUINT32 + 1.0));
   sum = pink->running_sum + new_random;
 
   /* Scale to range of -1.0 to 0.9999. */
@@ -680,7 +689,7 @@ gst_audio_test_src_create_pink_noise_##type (GstAudioTestSrc * src, g##type * sa
   while (i < (src->generate_samples_per_buffer * src->channels)) { \
     for (c = 0; c < src->channels; ++c) { \
       samples[i++] = \
-        (g##type) (gst_audio_test_src_generate_pink_noise_value (&src->pink) * \
+        (g##type) (gst_audio_test_src_generate_pink_noise_value (src) * \
         amp); \
     } \
   } \
@@ -797,8 +806,8 @@ gst_audio_test_src_create_gaussian_white_noise_##type (GstAudioTestSrc * src, g#
   \
   for (i = 0; i < src->generate_samples_per_buffer * src->channels; ) { \
     for (c = 0; c < src->channels; ++c) { \
-      gdouble mag = sqrt (-2 * log (1.0 - g_random_double ())); \
-      gdouble phs = g_random_double_range (0.0, M_PI_M2); \
+      gdouble mag = sqrt (-2 * log (1.0 - g_rand_double (src->gen))); \
+      gdouble phs = g_rand_double_range (src->gen, 0.0, M_PI_M2); \
       \
       samples[i++] = (g##type) (amp * mag * cos (phs)); \
       if (++c >= src->channels) \
@@ -849,9 +858,13 @@ gst_audio_test_src_change_wave (GstAudioTestSrc * src)
       src->process = silence_funcs[src->format];
       break;
     case GST_AUDIO_TEST_SRC_WAVE_WHITE_NOISE:
+      if (!(src->gen))
+        src->gen = g_rand_new ();
       src->process = white_noise_funcs[src->format];
       break;
     case GST_AUDIO_TEST_SRC_WAVE_PINK_NOISE:
+      if (!(src->gen))
+        src->gen = g_rand_new ();
       gst_audio_test_src_init_pink_noise (src);
       src->process = pink_noise_funcs[src->format];
       break;
@@ -864,6 +877,8 @@ gst_audio_test_src_change_wave (GstAudioTestSrc * src)
       src->process = tick_funcs[src->format];
       break;
     case GST_AUDIO_TEST_SRC_WAVE_GAUSSIAN_WHITE_NOISE:
+      if (!(src->gen))
+        src->gen = g_rand_new ();
       src->process = gaussian_white_noise_funcs[src->format];
       break;
     default:
@@ -941,8 +956,10 @@ gst_audio_test_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
   GstAudioTestSrc *src = GST_AUDIO_TEST_SRC (basesrc);
   GstClockTime time;
 
-  segment->time = segment->start;
+  GST_DEBUG_OBJECT (src, "seeking %" GST_SEGMENT_FORMAT, segment);
+
   time = segment->last_stop;
+  src->reverse = (segment->rate < 0.0);
 
   /* now move to the time indicated */
   src->next_sample =
@@ -951,8 +968,22 @@ gst_audio_test_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
   src->next_time =
       gst_util_uint64_scale_int (src->next_sample, GST_SECOND, src->samplerate);
 
+  GST_DEBUG_OBJECT (src, "seeking next_sample=%" G_GINT64_FORMAT
+      " next_time=%" GST_TIME_FORMAT, src->next_sample,
+      GST_TIME_ARGS (src->next_time));
+
   g_assert (src->next_time <= time);
 
+  if (!src->reverse) {
+    if (GST_CLOCK_TIME_IS_VALID (segment->start)) {
+      segment->time = segment->start;
+    }
+  } else {
+    if (GST_CLOCK_TIME_IS_VALID (segment->stop)) {
+      segment->time = segment->stop;
+    }
+  }
+
   if (GST_CLOCK_TIME_IS_VALID (segment->stop)) {
     time = segment->stop;
     src->sample_stop = gst_util_uint64_scale_int (time, src->samplerate,
@@ -993,7 +1024,7 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
   GstBuffer *buf;
   GstClockTime next_time;
   gint64 next_sample, next_byte;
-  guint bytes, samples;
+  gint bytes, samples;
   GstElementClass *eclass;
 
   src = GST_AUDIO_TEST_SRC (basesrc);
@@ -1014,8 +1045,10 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     src->tags_pushed = TRUE;
   }
 
-  if (src->eos_reached)
+  if (src->eos_reached) {
+    GST_INFO_OBJECT (src, "eos");
     return GST_FLOW_UNEXPECTED;
+  }
 
   /* if no length was given, use our default length in samples otherwise convert
    * the length in bytes to samples. */
@@ -1051,7 +1084,7 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
   } else {
     /* calculate full buffer */
     src->generate_samples_per_buffer = samples;
-    next_sample = src->next_sample + samples;
+    next_sample = src->next_sample + (src->reverse ? (-samples) : samples);
   }
 
   bytes = src->generate_samples_per_buffer * src->sample_size * src->channels;
@@ -1061,7 +1094,7 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
     return res;
   }
 
-  next_byte = src->next_byte + bytes;
+  next_byte = src->next_byte + (src->reverse ? (-bytes) : bytes);
   next_time = gst_util_uint64_scale_int (next_sample, GST_SECOND,
       src->samplerate);
 
@@ -1069,12 +1102,17 @@ gst_audio_test_src_create (GstBaseSrc * basesrc, guint64 offset,
   GST_LOG_OBJECT (src, "next_sample %" G_GINT64_FORMAT ", ts %" GST_TIME_FORMAT,
       next_sample, GST_TIME_ARGS (next_time));
 
-  GST_BUFFER_TIMESTAMP (buf) = src->timestamp_offset + src->next_time;
   GST_BUFFER_OFFSET (buf) = src->next_sample;
   GST_BUFFER_OFFSET_END (buf) = next_sample;
-  GST_BUFFER_DURATION (buf) = next_time - src->next_time;
+  if (!src->reverse) {
+    GST_BUFFER_TIMESTAMP (buf) = src->timestamp_offset + src->next_time;
+    GST_BUFFER_DURATION (buf) = next_time - src->next_time;
+  } else {
+    GST_BUFFER_TIMESTAMP (buf) = src->timestamp_offset + next_time;
+    GST_BUFFER_DURATION (buf) = src->next_time - next_time;
+  }
 
-  gst_object_sync_values (G_OBJECT (src), src->next_time);
+  gst_object_sync_values (G_OBJECT (src), GST_BUFFER_TIMESTAMP (buf));
 
   src->next_time = next_time;
   src->next_sample = next_sample;
