@@ -464,6 +464,49 @@ GST_START_TEST (test_refcount)
 
 GST_END_TEST;
 
+static void
+source_setup (GstElement * playbin, GstElement * source, GstElement ** p_src)
+{
+  GST_LOG ("source-setup called, source = %s", G_OBJECT_TYPE_NAME (source));
+  *p_src = gst_object_ref (source);
+  GST_LOG ("here");
+}
+
+GST_START_TEST (test_source_setup)
+{
+  GstElement *playbin, *videosink;
+  GstElement *src = NULL;
+
+  if (!gst_default_registry_check_feature_version ("redvideosrc", 0, 10, 0)) {
+    fail_unless (gst_element_register (NULL, "redvideosrc", GST_RANK_PRIMARY,
+            gst_red_video_src_get_type ()));
+  }
+
+  playbin = gst_element_factory_make ("playbin2", NULL);
+  g_object_set (playbin, "uri", "redvideo://", NULL);
+
+  videosink = gst_element_factory_make ("fakesink", "myvideosink");
+  g_object_set (playbin, "video-sink", videosink, NULL);
+
+  g_signal_connect (playbin, "source-setup", G_CALLBACK (source_setup), &src);
+
+  fail_unless_equals_int (gst_element_set_state (playbin, GST_STATE_PAUSED),
+      GST_STATE_CHANGE_ASYNC);
+  fail_unless_equals_int (gst_element_get_state (playbin, NULL, NULL,
+          GST_CLOCK_TIME_NONE), GST_STATE_CHANGE_SUCCESS);
+
+  fail_unless (src != NULL);
+  fail_unless (G_OBJECT_TYPE (src) == gst_red_video_src_get_type ());
+
+  fail_unless_equals_int (gst_element_set_state (playbin, GST_STATE_NULL),
+      GST_STATE_CHANGE_SUCCESS);
+
+  gst_object_unref (playbin);
+  gst_object_unref (src);
+}
+
+GST_END_TEST;
+
 /*** redvideo:// source ***/
 
 static GstURIType
@@ -475,7 +518,7 @@ gst_red_video_src_uri_get_type (void)
 static gchar **
 gst_red_video_src_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { "redvideo", NULL };
+  static gchar *protocols[] = { (char *) "redvideo", NULL };
 
   return protocols;
 }
@@ -522,8 +565,6 @@ GST_BOILERPLATE_FULL (GstRedVideoSrc, gst_red_video_src, GstPushSrc,
 static void
 gst_red_video_src_base_init (gpointer klass)
 {
-  static const GstElementDetails details =
-      GST_ELEMENT_DETAILS ("Red Video Src", "Source/Video", "yep", "me");
   static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
       GST_PAD_SRC, GST_PAD_ALWAYS,
       GST_STATIC_CAPS ("video/x-raw-yuv, format=(fourcc)I420")
@@ -532,7 +573,8 @@ gst_red_video_src_base_init (gpointer klass)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_templ));
-  gst_element_class_set_details (element_class, &details);
+  gst_element_class_set_details_simple (element_class,
+      "Red Video Src", "Source/Video", "yep", "me");
 }
 
 static GstFlowReturn
@@ -585,7 +627,7 @@ gst_codec_src_uri_get_type (void)
 static gchar **
 gst_codec_src_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { "codec", NULL };
+  static gchar *protocols[] = { (char *) "codec", NULL };
 
   return protocols;
 }
@@ -635,8 +677,6 @@ GST_BOILERPLATE_FULL (GstCodecSrc, gst_codec_src, GstPushSrc,
 static void
 gst_codec_src_base_init (gpointer klass)
 {
-  static const GstElementDetails details =
-      GST_ELEMENT_DETAILS ("Codec Src", "Source/Video", "yep", "me");
   static GstStaticPadTemplate src_templ = GST_STATIC_PAD_TEMPLATE ("src",
       GST_PAD_SRC, GST_PAD_ALWAYS,
       GST_STATIC_CAPS ("application/x-codec")
@@ -645,7 +685,8 @@ gst_codec_src_base_init (gpointer klass)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_templ));
-  gst_element_class_set_details (element_class, &details);
+  gst_element_class_set_details_simple (element_class,
+      "Codec Src", "Source/Video", "yep", "me");
 }
 
 static GstFlowReturn
@@ -698,6 +739,7 @@ playbin2_suite (void)
   tcase_add_test (tc_chain, test_missing_suburisource_handler);
   tcase_add_test (tc_chain, test_missing_primary_decoder);
   tcase_add_test (tc_chain, test_refcount);
+  tcase_add_test (tc_chain, test_source_setup);
 
   /* one day we might also want to have the following checks:
    * tcase_add_test (tc_chain, test_missing_secondary_decoder_one_fatal);
