@@ -204,11 +204,10 @@ print_stream_info (GstDiscovererStreamInfo * info, void *depth)
       desc =
           gst_stream_video_information_to_string (info,
           GPOINTER_TO_INT (depth) + 1);
-  }
-
-  if (desc) {
-    g_print ("%s", desc);
-    g_free (desc);
+    if (desc) {
+      g_print ("%s", desc);
+      g_free (desc);
+    }
   }
 }
 
@@ -229,7 +228,9 @@ print_topology (GstDiscovererStreamInfo * info, gint depth)
   } else if (GST_IS_DISCOVERER_CONTAINER_INFO (info)) {
     GList *tmp, *streams;
 
-    streams = gst_discoverer_container_info_get_streams (info);
+    streams =
+        gst_discoverer_container_info_get_streams (GST_DISCOVERER_CONTAINER_INFO
+        (info));
     for (tmp = streams; tmp; tmp = tmp->next) {
       GstDiscovererStreamInfo *tmpinf = (GstDiscovererStreamInfo *) tmp->data;
       print_topology (tmpinf, depth + 1);
@@ -238,11 +239,41 @@ print_topology (GstDiscovererStreamInfo * info, gint depth)
   }
 }
 
-static void
-print_duration (GstDiscovererInfo * info, gint tab)
+static gboolean
+print_tag_each (GQuark field_id, const GValue * value, gpointer user_data)
 {
-  g_print ("%*s%" GST_TIME_FORMAT "\n", tab + 1, " ",
+  gint tab = GPOINTER_TO_INT (user_data);
+  gchar *ser;
+
+  if (G_VALUE_HOLDS_STRING (value))
+    ser = g_value_dup_string (value);
+  else if (GST_VALUE_HOLDS_BUFFER (value)) {
+    GstBuffer *buf = gst_value_get_buffer (value);
+    ser = g_strdup_printf ("<GstBuffer [%d bytes]>", GST_BUFFER_SIZE (buf));
+  } else
+    ser = gst_value_serialize (value);
+
+  g_print ("%*s%s: %s\n", tab, " ",
+      gst_tag_get_nick (g_quark_to_string (field_id)), ser);
+  g_free (ser);
+
+  return TRUE;
+}
+
+static void
+print_properties (GstDiscovererInfo * info, gint tab)
+{
+  const GstTagList *tags;
+
+  g_print ("%*sDuration: %" GST_TIME_FORMAT "\n", tab + 1, " ",
       GST_TIME_ARGS (gst_discoverer_info_get_duration (info)));
+  g_print ("%*sSeekable: %s\n", tab + 1, " ",
+      (gst_discoverer_info_get_seekable (info) ? "yes" : "no"));
+  if ((tags = gst_discoverer_info_get_tags (info))) {
+    g_print ("%*sTags: \n", tab + 1, " ");
+    gst_structure_foreach ((const GstStructure *) tags, print_tag_each,
+        GINT_TO_POINTER (tab + 5));
+  }
 }
 
 static void
@@ -294,8 +325,8 @@ print_info (GstDiscovererInfo * info, GError * err)
   if ((sinfo = gst_discoverer_info_get_stream_info (info))) {
     g_print ("\nTopology:\n");
     print_topology (sinfo, 1);
-    g_print ("\nDuration:\n");
-    print_duration (info, 1);
+    g_print ("\nProperties:\n");
+    print_properties (info, 1);
     gst_discoverer_stream_info_unref (sinfo);
   }
 
@@ -460,6 +491,7 @@ main (int argc, char **argv)
 
     gst_discoverer_stop (dc);
     g_free (ps);
+    g_main_loop_unref (ml);
   }
   g_object_unref (dc);
 
